<?php

namespace Drupal\private_message\Drush\Commands;

use Drupal\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\private_message\Service\PrivateMessageService;
use Drush\Attributes as CLI;
use Drush\Commands\DrushCommands;

/**
 * Drush commands for the Private Message module.
 */
class PrivateMessageCommands extends DrushCommands {

  use StringTranslationTrait;

  public function __construct(
    protected readonly PrivateMessageService $privateMessageService,
    protected readonly EntityTypeManagerInterface $entityTypeManager,
  ) {
    parent::__construct();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('private_message.service'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * Prepares the Private Message module for uninstallation.
   */
  #[CLI\Command('private_message:prepare_uninstall', aliases: ['pu'])]
  public function prepareUninstall(): void {
    if ($this->io()->confirm($this->t('Proceeding will delete all private messages and private message threads in your system. They cannot be recovered. Do you wish to proceed?')->render())) {
      // Get the IDs of all threads in the system.
      $thread_ids = $this->privateMessageService->getThreadIds();
      // Load the private message thread storage.
      $thread_storage = $this->entityTypeManager->getStorage('private_message_thread');
      $message_storage = $this->entityTypeManager->getStorage('private_message');

      $output = $this->output();
      // Note that each thread ID is looped through, as there may be a massive
      // number of threads in the system, meaning that loading them all could be
      // a crazy memory hog and crash.
      foreach ($thread_ids as $thread_id) {
        // Load the thread.
        $thread = $thread_storage->load($thread_id);
        $message_storage->delete($thread->getMessages());
        // Delete the thread.
        $thread_storage->delete([$thread]);
        // Notify the user.
        $output->writeln($this->t('Deleted thread: @id', ['@id' => $thread_id])->render());
      }
      // Inform the user the process is finished.
      $output->writeln($this->t('All private message content deleted.')->render());
    }
  }

}
